@@ -148,10 +148,10 @@ class bdist_esky(Command):
                          freezer module.
 
         bootstrap_module:  a custom module to use for esky bootstrapping;
-                           the default calls esky.bootstrap.common.bootstrap()
+                           the default calls esky.bootstrap.common.bootstrap().
 
         bootstrap_code:  a custom code string to use for esky bootstrapping;
-                         the default calls esky.bootstrap.common.bootstrap()
+                         this precludes the use of the bootstrap_module option.
 
         dont_run_startup_hooks:  don't force all executables to call
                                  esky.run_startup_hooks() on startup.
@@ -160,6 +160,16 @@ class bdist_esky(Command):
                         as a private assembly.  The default is False; only
                         those with a valid license to redistriute these files
                         should enable it.
+
+
+
+        pre_freeze_callback:  function to call just before starting to freeze
+                              the application; this is a good opportunity to
+                              customize the bdist_esky instance.
+
+        pre_zip_callback:  function to call just before starting to zip up
+                           the frozen application; this is a good opportunity
+                           to e.g. sign the resulting executables.
     
     """
 
@@ -198,6 +208,8 @@ class bdist_esky(Command):
         self.dont_run_startup_hooks = False
         self.bootstrap_module = None
         self.bootstrap_code = None
+        self.pre_freeze_callback = None
+        self.pre_zip_callback = None
 
     def finalize_options(self):
         self.set_undefined_options('bdist',('dist_dir', 'dist_dir'))
@@ -223,7 +235,18 @@ class bdist_esky(Command):
                     err = err % (self.freezer_module,)
                     raise RuntimeError(err)
             self.freezer_module = freezer
+        if isinstance(self.pre_freeze_callback,basestring):
+            self.pre_freeze_callback = self._name2func(self.pre_freeze_callback)
+        if isinstance(self.pre_zip_callback,basestring):
+            self.pre_zip_callback = self._name2func(self.pre_zip_callback)
 
+    def _name2func(self,name):
+        """Convert a dotted name into a function reference."""
+        if "." not in name:
+            return globals()[name]
+        modname,funcname = name.rsplit(".",1)
+        mod = __import__(modname,fromlist=[funcname])
+        return getattr(mod,funcname)
 
     def run(self):
         self.tempdir = tempfile.mkdtemp()
@@ -233,7 +256,16 @@ class bdist_esky(Command):
             shutil.rmtree(self.tempdir)
 
     def _run(self):
-        #  Create the dirs into which to freeze the app
+        self._run_initialise_dirs()
+        if self.pre_freeze_callback is not None:
+            self.pre_freeze_callback(self)
+        self._run_freeze_scripts()
+        if self.pre_zip_callback is not None:
+            self.pre_zip_callback(self)
+        self._run_create_zipfile()
+
+    def _run_initialise_dirs(self):
+        """Create the dirs into which to freeze the app."""
         fullname = self.distribution.get_fullname()
         platform = get_platform()
         self.bootstrap_dir = os.path.join(self.dist_dir,
@@ -243,9 +275,12 @@ class bdist_esky(Command):
         if os.path.exists(self.bootstrap_dir):
             shutil.rmtree(self.bootstrap_dir)
         os.makedirs(self.freeze_dir)
-        #  Hand things off to the selected freezer module
+
+    def _run_freeze_scripts(self):
+        """Call the selected freezer module to freeze the scripts."""
+        fullname = self.distribution.get_fullname()
+        platform = get_platform()
         self.freezer_module.freeze(self)
-        #  Create the necessary control files
         if platform != "win32":
             lockfile = os.path.join(self.freeze_dir,ESKY_CONTROL_DIR,"lockfile.txt")
             with open(lockfile,"w") as lf:
@@ -254,15 +289,18 @@ class bdist_esky(Command):
             shutil.copyfile(os.path.join(self.freeze_dir,ESKY_CONTROL_DIR,"lockfile.txt"),os.path.join(self.freeze_dir,"esky-lockfile.txt"))
         # TODO: remove compatability hook
         shutil.copyfile(os.path.join(self.freeze_dir,ESKY_CONTROL_DIR,"bootstrap-manifest.txt"),os.path.join(self.freeze_dir,"esky-bootstrap.txt"))
-        #  Zip up the distribution
+
+    def _run_create_zipfile(self):
+        """Zip up the final distribution."""
         print "zipping up the esky"
+        fullname = self.distribution.get_fullname()
+        platform = get_platform()
         zfname = os.path.join(self.dist_dir,"%s.%s.zip"%(fullname,platform,))
         if hasattr(self.freezer_module,"zipit"):
             self.freezer_module.zipit(self,self.bootstrap_dir,zfname)
         else:
             create_zipfile(self.bootstrap_dir,zfname,compress=True)
         shutil.rmtree(self.bootstrap_dir)
-        
 
     def get_executables(self,rewrite=True):
         """Yield an Executable instance for each script to be frozen.
