@@ -73,15 +73,19 @@ def freeze(dist):
     if sys.platform == "win32":
         code_source.append(_CUSTOM_WIN32_CHAINLOADER)
     code_source.append("__esky_name__ = '%s'" % (dist.distribution.get_name(),))
-    if dist.bootstrap_module is None:
-        code_source.append("bootstrap()")
-    else:
+    if dist.bootstrap_code is not None:
+        code_source.append("__name__ = '__main__'")
+        code_source.append(dist.bootstrap_code)
+        code_source.append("raise RuntimeError('didnt chainload')")
+    elif dist.bootstrap_module is not None:
         code_source.append("__name__ = '__main__'")
         bsmodule = __import__(dist.bootstrap_module)
         for submod in dist.bootstrap_module.split(".")[1:]:
             bsmodule = getattr(bsmodule,submod)
         code_source.append(inspect.getsource(bsmodule))
         code_source.append("raise RuntimeError('didnt chainload')")
+    else:
+        code_source.append("bootstrap()")
     code_source = "\n".join(code_source)
     maincode = imp.get_magic() + struct.pack("<i",0)
     maincode += marshal.dumps(compile(code_source,"__main__.py","exec"))
